// -*- C++ -*-

//
//  kjots
//
//  Copyright (C) 1997 Christoph Neerfeld
//  Copyright (C) 2002, 2003 Aaron J. Seigo
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#ifndef KJotsMain_included
#define KJotsMain_included

#include <qlayout.h>
#include <qptrlist.h>
#include <qstringlist.h>
#include <qdialog.h>
#include <qlineedit.h>
#include <qpushbutton.h>

#include <kdialogbase.h>
#include <kmainwindow.h>
#include <ktextedit.h>
#include <klistview.h>

class QGroupBox;
class KAccel;
class KJotsMain;
class KToolBar;
class KListBox;
class KListView;
class KPopupMenu;
class ConfigureDialog;
class KEdFind;
class KEdReplace;

class KJotsEntry;

/////////////////////////////////////////////////////////////////////////////
class MyMultiEdit : public KTextEdit
{
  Q_OBJECT
public:
  MyMultiEdit (QWidget* parent=0, const char* name=0);
  ~MyMultiEdit ();

  void search();
  void repeatSearch();
  void replace() {}
  void print(QString title = QString::null);

signals:
  void findSuccessful();

protected slots:
  void openUrl();
  void slotSearch();

protected:
  virtual void mousePressEvent (QMouseEvent *e);

  KPopupMenu *web_menu;

  // search / replace dialogs
  KEdFind       *mSearchDialog;
  KEdReplace *mReplaceDialog;
  QString           mSearchFor;
};


/////////////////////////////////////////////////////////////////////////////
class KJotsMain : public KMainWindow
{
  Q_OBJECT
public:
  KJotsMain( const char* name = NULL );
  virtual ~KJotsMain();

public slots:
  void saveFolder();

protected slots:
  // called once at the start of the app with a single shot timer
  void slotSetupInterface();
  void openFolder(int id);
  void createFolder();
  void deleteFolder();
  void nextEntry();
  void prevEntry();
  void jumpToEntry(QListViewItem* entry);
  void newEntry();
  void deleteEntry();
  void addToHotlist();
  void removeFromHotlist();
  void configure();
  void configureKeys();
  void configureToolbars();
  void updateToolbars();
  void configureHide();
  void configureDestroy();
  void updateConfiguration();
  void writeBook();
  void writePage();
  void copySelection();
  void insertDate();
  void slotEditBookTitle();
  void slotPrintPage();
  void slotSearch();
  void slotRepeatSearch();
  void slotReplace();
  void slotQuit();
  void slotShowHideToolbars();
  void slotFindSuccessful();

  // handling page changes in the listview
  void showListviewContextenu(KListView* l, QListViewItem* i, const QPoint& p);
  void renameCurrentEntry();

protected:
  int readFile(QString name);
  int writeFile( QString name);
  void writeEntry( QTextStream &st, KJotsEntry *entry );
  void saveProperties( void );

  // save the current text/title and move to a new item
  // does no error checking as that should be done before calling this
  void prepTextArea(KJotsEntry* entry);

  // turn actions on/off
  void updateMenu();

  int                 currentFolder;
  bool             folderOpen;
  QString          current_folder_name;
  QStringList         folder_list;
  QStringList         hotlist;
  QPtrList<QPushButton> button_list;
  QPopupMenu      *folders_menu;
  int              unique_id;
  ConfigureDialog *mOptionDialog;

  QHBoxLayout   *bg_top_layout;
  QButtonGroup  *bg_top;
  MyMultiEdit   *me_text;
  KListView     *subjectList;
  KJotsEntry    *previousEntry;

  int id_fav_add;
  int id_fav_del;
};

#endif // KJotsMain_included
